/**
 * 
 * local statistics - BROWSER INTERNAL TRACKING RESPECTING USERS PRIVACY<br />
 * <br />
 * This is a page tracker for a website.<br />
 * It stores the visited pages and count of their visits in the local storage<br />
 * of the webbrowser of your visitor. All Tracking data stay at the user.<br />
 * It does NOT use any cookie (a cookie will be sent to the webserver). <br />
 * <br />
 * You can understand it as a tracker that respects the users privacy.<br />
 * <br />
 * It means:<br />
 * - the owner of the website has NO control about these statistic data<br />
 * - the user can delete the private data of the browser resets the counter<br />
 * BUT:<br />
 * as owner of a website you can <br />
 * - let count the visits on the client side<br />
 * - react i.e if the user visits yout page the 100th time<br />
 * - let draw local statistics on users request<br />
 * <br />
 * PROJECT HOME: <a href="http://www.axel-hahn.de/projects/javascript/localstats/" target="_blank">http://www.axel-hahn.de/projects/javascript/localstats/</a><br />
 * DOC:          <a href="http://www.axel-hahn.de/docs/localstats/index.htm" target="_blank">http://www.axel-hahn.de/docs/localstats/index.htm</a><br />
 * <br />
 * TODO<br />
 * - override default options in the constructor<br />
 * - let the user skip all local trackings<br />
 * 
 * @author    Axel Hahn
 * @version   0.02
 *
 * @this {localStats}
 * 
 * @constructor
 * 
 * @example
 * &lt;script src="/javascript/localstats.class.js">&lt;/script>
 * (...)
 * &lt;script>
 *   var oStats=new localStats();
 *   
 *   // track the request
 *   oStats.track();
 *   
 *   // show counter
 *   document.getElementById("spanCounter").innerHTML=oStats.getCountTotal();
 * &lt;/script>
 * 
 * @returns {localStats}
 */
var localStats=function (){
    
    // ------------------------------------------------------------
    // config
    // ------------------------------------------------------------
    
    // name of the local storage variable
    this._sStatsVarName="yourOwnLocalPageCounter";
    
    // default options
    this._aOptions={
        'iTableTop':10,            // default: Top [n] table if no param was set at renderTopTable()
        'trackQueryParams':false,  // track GET params? default is NO
        
        'blacklist':[
            'index.php',
            'secret_directory_or_whatever'
        ],
        
        // TODO: these parameters are just ideas so far
        'skipTracking':false      // allow a user to skip tracking inside its own webbrowser
    };
    
    // ------------------------------------------------------------
    // init
    // ------------------------------------------------------------
    this._aPageViews=JSON.parse(localStorage.getItem(this._sStatsVarName));    
    if (arguments) {
        for (var s in arguments[0]) {
            this._aOptions[s]=arguments[0][s];
        }
    }


    // ------------------------------------------------------------
    // tracking
    // ------------------------------------------------------------

    /**
     * track the the current page.
     * it repects the values of trackQueryParams and blacklist in this._aOptions 
     * @returns {Boolean}
     */
    this.track = function () {
        var sUrlToTrack=window.location.pathname;
        
        // check query params
        if (this._aOptions['trackQueryParams']){
            sUrlToTrack+=window.location.search;
        }
        
        // check blacklist
        if (this._aOptions && this._aOptions['blacklist'] && this._aOptions['blacklist'].length){
            for (var i=0; i<this._aOptions['blacklist'].length; i++){
                var regex=this._aOptions['blacklist'][i];
                if (sUrlToTrack.match(regex)){
                    return false;
                }
            }
        }
        
        var aPageViews=this._aPageViews;
        if(!aPageViews){
            // build a default object
            aPageViews=new Object();
            aPageViews["__total"]=0;
            aPageViews["__remark"]='Was das hier ist? In deinem Webbrowser - und nur dort und bei dir lokal(!) - wird die Anzahl der Seitenaufrufe gespeichert als auch die Anzahl der Aufrufe der Einzelseiten. Bei Gelegenheit gibt es eine persoenliche Statistik-Seite. Bei Fragen oder Bedenken kontatiere mich per E-Mail (s. Fusszeile).';
            aPageViews["__first_visit"]=new Date();
            aPageViews["pages"]=new Object();
        }
        if (!aPageViews["pages"][sUrlToTrack]){
            aPageViews["pages"][sUrlToTrack]=0;
        }
        aPageViews["pages"][sUrlToTrack]++;
        aPageViews["__total"]++;
        localStorage.setItem(this._sStatsVarName, JSON.stringify(aPageViews));
        return true;
    };
    
    // ------------------------------------------------------------
    // flush data
    // ------------------------------------------------------------

    /**
     * flush all local statistic data
     * @returns {boolean}
     */
    this.flush = function () {
        return localStorage.removeItem(this._sStatsVarName);
    };

    // ------------------------------------------------------------
    // fetch results
    // ------------------------------------------------------------
    
    /**
     * get count of tracked requests
     * @see getPageCount()
     * @returns {integer}
     */
    this.getCountTotal = function () {
        if (!this._aPageViews) return false;
        return this._aPageViews["__total"];
    };
    
    /**
     * get date object of the first tracked visit
     * @returns {object}
     */
    this.getCountSince = function () {
        if (!this._aPageViews) return false;
        return this._aPageViews["__first_visit"];
    };

    /**
     * get the count of different pages that have been requested
     * @see getCountTotal()
     * @returns {Number|Boolean}
     */
    this.getPageCount = function () {
        if (!this._aPageViews) return false;
        var iCounter=0;
        for (var s in this._aPageViews["pages"]){
            iCounter++;
        }
        return iCounter;
    };


    /**
     * private function.
     * helper function to sort the requested pages by count of their requests
     * ordered descend.
     * @param {array} a
     * @param {array} b
     * @see getStats()
     * @returns {integer}
     */
    this._compare = function (a, b) {
        if (a[1] > b[1]) { return -1; }
        else { return (a[1] < b[1]) ? 1 : 0;}
    };

    /**
     * get array of reqested pages ordered by count of requests
     * @returns {Array}
     */
    this.getStats = function () {
        if (!this._aPageViews) return false;

        var aValues=new Array();
        for (var s in this._aPageViews["pages"]){
            aValues.push([s, this._aPageViews["pages"][s]/1]);
        }
        aValues.sort(this._compare);
        return aValues;
    };

    /**
     * get count of most requested page
     * @see getCountTotal()
     * @returns {Number|Boolean}
     */
    this.getMaxCountOfAPage = function () {
        if (!this._aPageViews) return false;
        return this.getStats()[0][1];
    };

    /**
     * private function.
     * get html code for a table row of a stats table
     * @param {Array}   aItem  array with [ page, count ]
     * @see renderTopTable()
     * @returns {String}
     */
    this._renderTopTableItem = function(aItem){
        var sUrl=aItem[0];
        var iCount=aItem[1];
        var iMax2=this.getMaxCountOfAPage();
        var iPercent=Math.round(iCount/this.getCountTotal()*100);
        var iWidth=Math.round(iCount/iMax2*100);
        var sHtml='<tr><td>'+sUrl+'</td><td>'+iCount+'</td><td>'+iPercent+'</td><td><div class="localstatsbar" style="width: '+iWidth +'%;">&nbsp;</div></td></tr>';
        return sHtml;
    };

    /**
     * get html code for a top [n] table 
     * @param {Number} iTop
     * @returns {String}
     */
    this.renderTopTable = function(iTop){
        if (!iTop){
            iTop=this._aOptions['iTableTop'];
        }
        var sHtml='';
        var aData=this.getStats();
        if(aData && aData.length){
            var iCount=(aData.length>iTop)?iTop:aData.length;
            sHtml='<h4>Top '+iTop+' Deiner Seitenaurufe</h4>\n\
                <table>\n\
                    <tr><th>Seite</td><th>Aufrufe</th><th>Prozent</th><th style="width: 200px;">Graph</th></tr>';
            for (var i=0; i<iCount; i++){
                sHtml+=this._renderTopTableItem(aData[i], aData[0][1]);
            }
            sHtml+='</table>';
        }
        return sHtml;
    };
    
};
